require 'mph'
require 'mph/traditional'

module RPM
  class Package
    attr_reader :install

    def mark_install
      if !@can_not_install then
        @install = true
      else
        false
      end
    end

    def unmark_install
      @install = false
      @can_not_install = true
    end

    def need_to_download(cachedir)
      if @filename != "" then
        begin
          p = RPM::Package.open(cachedir + @filename)
          if self == p then
            false
          else
            true
          end
        rescue
          true
        end
      else
        false
      end
    end

    def get_file
      if @location[0..0] == '/'
        true
      else
        MPH::get_remote_file(@location + "/" + @filename)
      end
    end
  end
end

module MPH
  class Mph < MPH::DB
    attr_reader :need_to_install

    def get_all_packages(test_mode = false)
      begin
        Dir.chdir(@sCacheDir)
      rescue Errno::ENOENT
        File.mkpath(@sCacheDir)
        retry
      end
      ret = true
      @need_to_download.each do |p|
        if !test_mode then
          ret &= p.get_file
        end
      end
      if @force_mode then
        ret = true
      end
      ret
    end

    def print_download_info_version_check(pkgs)
      i = 0
      localdb = RPM::DB.new_with_wait
      pkgs.sort.each do |rp|
	mi = localdb.init_iterator(RPM::DBI_LABEL,rp.name)
        if mi then
          lp = mi.next_iterator
          if lp.is_kernel? then
            @msg.print(MsgLevel::NORMAL, "I #{rp.name} : #{lp.version} -> #{rp.version}\n")
          else
            @msg.print(MsgLevel::NORMAL, "U #{rp.name} : #{lp.version} -> #{rp.version}\n")
          end
        else
          @msg.print(MsgLevel::NORMAL, "N #{rp.name} : #{rp.version}\n")
        end
        i += rp.size
      end
      i
    end
    private :print_download_info_version_check

    def print_download_info
      if @cached.length > 0 then
        @msg.print(MsgLevel::NORMAL, "Following packages are cached: \n" )
        print_download_info_version_check(@cached)
        @msg.print(MsgLevel::NORMAL, "\n" )
      end
      if @need_to_download.length > 0 then
        @msg.print(MsgLevel::NORMAL, "Following packages will be downloaded: \n" )
        i = print_download_info_version_check(@need_to_download)
        @msg.print(MsgLevel::NORMAL, "Total \%.1fMbyte will be downloaded.\n" % (i / 1024.0 / 1024.0 ))
        @msg.print(MsgLevel::NORMAL, "\n" )
      end
    end

    #ȤꤢޥȤƥ֤ä
    def install(test_mode = false)
      if test_mode then
	return
      end
      cb = nil
      if block_given? then
        cb = Proc.new
      end
     
	GC.start
      install_packages = []
      install_packages_kernel = []
      installed = 0
      @need_to_install.each do |p|
        installed += 1
        if p.is_kernel? then
          install_packages_kernel.push(p.filename)
        else
          install_packages.push(p.filename)
        end
      end
      Dir.chdir(@sCacheDir)
      unless install_packages_kernel.empty? then
        system("rpm -ivh #{install_packages_kernel.join(' ')}")
        raise("An error occured.") unless $?.to_i == 0
        unless @keep_local_copy then
          `rm -f #{install_packages_kernel.join(' ')}`
        end
      end
      unless install_packages.empty? then
        system("rpm -Uvh --oldpackage --replacefiles --replacepkgs #{install_packages.join(' ')}")
        raise("An error occured.") unless $?.to_i == 0
        unless @keep_local_copy then
          `rm -f #{install_packages.join(' ')}`
        end
      end
      @msg.print(MsgLevel::NORMAL, "#{installed} packages are installed.\n")
    end

    def print_log(what,p,d,additional=nil)
      @msg.print(MsgLevel::LOG,"  #{what} #{p}\n")
      @msg.print(MsgLevel::LOG,"   why #{d}\n")
      if additional then
        @msg.print(MsgLevel::LOG,"   #{additional}\n")
      end
    end
    private :print_log

    def proc_require(deps,localdb)
      @msg.print(MsgLevel::LOG,"  proc_require: deps:\n")
        deps.each do |d|
        @msg.print(MsgLevel::LOG,"    #{d}\n")
      end
      deps.delete_if do |d|
        found = false
        if d.is_a?(RPM::Require) then
          if @provides_cache[d.name] then
            @provides_cache[d.name].each do |p|
              if d.satisfy?(p) then
                if p.mark_install then
                  print_log("install1",p,d)
                  found = true
                  break
                end
              end
            end
          end
          if !found && @mphPackages[d.owner.name] then
            @mphPackages[d.owner.name].each do |p|
              if p.signature != d.owner.signature then
                # rpm bug?..
                if p.version == d.owner.version then
		  mi = localdb.init_iterator(RPM::TAG_PROVIDENAME,d.name)
                  if mi then
                    print_log("remove(rpm same version problem?..)",p,d)
                    p.unmark_install
                    mi.each do |lp|
                      if @mphPackages[lp.name] then
                        @mphPackages[lp.name].each do |rp|
                          print_log("remove(rpm same version problem?..)",rp,d)
                          rp.unmark_install
                        end
                      end
                    end
                    found = true
                    break
                  end
                end
                if p.mark_install then
                  print_log("install2",@mphPackages[d.owner.name][0],d)
                  found = true
                  break
                end
              end
            end
          end
          if !found && @obsoletes_cache[d.owner.name] then
            if @obsoletes_cache[d.owner.name][0].mark_install then
              p = @obsoletes_cache[d.owner.name][0]
              print_log("install3",p,d,"#{d.owner.name} obsoleted by #{p}")
              found = true
            end
          end
        end
        found
      end
    end
    private :proc_require

    def proc_remove(deps,localdb)
      @msg.print(MsgLevel::LOG,"  proc_remove: deps:\n")
      deps.each do |d|
        @msg.print(MsgLevel::LOG,"    #{d}\n")
      end
      deps.delete_if do |d|
        found = false
        if @mphPackages[d.owner.name] then
          @mphPackages[d.owner.name].each do |p|
            if p.sigmd5 == d.owner.sigmd5 then
              print_log("remove1",p,d)
              p.unmark_install
              found = true
              break
            end
          end
	else
	  mi = localdb.init_iterator(RPM::TAG_PROVIDENAME,d.name)
	  if mi then
	    mi.each do |lp|
	      if @mphPackages[lp.name] then
		@mphPackages[lp.name].each do |p| 
		  print_log("remove2",p,d)
		  p.unmark_install
		  found = true
		  break
		end
	      end
	    end
	  end
        end
        found
      end
    end
    private :proc_remove

    def proc_conflicts(deps,localdb)
      @msg.print(MsgLevel::LOG,"  proc_conflicts: deps:\n")
      deps.each do |d|
        @msg.print(MsgLevel::LOG,"    #{d}\n")
      end
      deps.delete_if do |d|
        found = false
        if d.is_a?(RPM::Conflict) then
          if (d.flags != 0) && @mphPackages[d.name] then
            @mphPackages[d.name].each do |p|
              if !d.satisfy?(p) && found = p.mark_install then
                print_log("install4",p,d)
              end
            end
          end
          if !found then
            if @obsoletes_cache[d.owner.name] then
              if @obsoletes_cache[d.owner.name][0].mark_install then
                p = @obsoletes_cache[d.owner.name][0]
                print_log("install5",p,d,"#{d.owner.name} obsoleted by #{p}")
                found = true
                break
              end
            end
          end
        end
        found
      end
    end
    private :proc_conflicts

    def do_resolve(localdb)
      deps = nil
      localdb.transaction do |ts|
        each do |rp|
          if rp.install then
            ts.upgrade(rp,rp.to_s)
          end
        end
        deps = ts.check
        ts.abort
      end
      if deps then
        proc_require(deps,localdb)
        proc_conflicts(deps,localdb)
        proc_remove(deps,localdb)
      end
      deps
    end
    private :do_resolve

    def check(localdb)
      @msg.print(MsgLevel::NORMAL,'.')
      count=1
      @msg.print(MsgLevel::LOG,"do_resolve phase #{count}\n")
      deps = do_resolve(localdb)
      count+=1
      while deps
        if deps.length != 0 then
          @msg.print(MsgLevel::ERROR,"\nFollowing dependencies are unresolved: \n")
          deps.each do |d|
            @msg.print(MsgLevel::ERROR,"#{d}\n")
          end
          @msg.print(MsgLevel::ERROR," give up...\n")
          return false
        else
          @msg.print(MsgLevel::NORMAL,'.')
          @msg.print(MsgLevel::LOG,"do_resolve phase #{count}\n")
          deps = do_resolve(localdb)
          count += 1
        end
      end
      @msg.print(MsgLevel::NORMAL,"\n")
      true
    end
    private :check

    def resolve(provides = nil)
      @msg.print(MsgLevel::NORMAL, "Resolving dependencies" )

      localdb = RPM::DB.new_with_wait(true)
      if !provides then
        each do |p|
          if localdb.init_iterator(RPM::DBI_LABEL,p.name) then
            p.mark_install
          end
        end
      else
        provides.each do |provide_name|
          provider = nil
          each_match(RPM::DBI_LABEL,provide_name) do |pkg|
            if !provider then
              provider = pkg
            elsif pkg.version > provider.version
              provider = pkg
            end
          end
          if !provider && !( provide_name =~ /kernel/ ) then
            each_match(RPM::TAG_PROVIDENAME,provide_name) do |pkg|
              provider = pkg
              break
            end
          end
          if provider then
            provider.mark_install
          end
        end
      end

      if !check(localdb) then
        exit -1
      end
      localdb = nil
      @msg.print(MsgLevel::NORMAL,"Done.\n")

      @mphPackages.each do |k,v|
        v.each do |p|
          if p.install then
            @need_to_install << p
            if p.need_to_download(@sCacheDir) then
              @need_to_download << p
            else
              @cached << p
            end
          end
        end
      end
      if @need_to_install.length == 0 then
        @msg.print(MsgLevel::NORMAL, "No package was updated.\n" )
      end
      true
    end

    def setup_db(dont_get_dot_mph = false,dont_use_hold = false,with_kernel = false)
      if Zlib then
        super([DBLoader,TraditionalDBLoader],dont_get_dot_mph,dont_use_hold,with_kernel)
      else
        super([TraditionalDBLoader],dont_get_dot_mph,dont_use_hold,with_kernel)
      end
    end

    def initialize(sConfFile='/etc/mph.conf', force_mode = false)
      super
      @need_to_install = []
      @need_to_download = []
      @cached = []
    end
  end
end
