#!/usr/bin/ruby

#//------------------------------------------------
#  Copyright (C) 2000 Tenkou N. Hattori <tnh@kondara.org>
#  Copyright (C) 2000 Kondara Project <http://www.kondara.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#//------------------------------------------------

require 'setlk'
require 'mph/traditional'

module MPH
  class Scanpackages
    def self.create_bone(rpmfile)
      if rpmfile.is_a? RPM::Package
        source = rpmfile
      else
        source = RPM::Package.open(rpmfile)
      end
      p = RPM::Package.create(source.name,source.version)
      if source.requires
        source.requires.each do |d|
          p.add_dependency(d)
        end
      end
      if source.provides
        source.provides.each do |d|
          p.add_dependency(d)
        end
      end
      if source.conflicts
        source.conflicts.each do |d|
          p.add_dependency(d)
        end
      end
      if source.obsoletes
        source.obsoletes.each do |d|
          p.add_dependency(d)
        end
      end
      p.add_binary(RPM::TAG_SIGMD5,source[RPM::TAG_SIGMD5])
      p.add_string(RPM::TAG_SOURCERPM,source[RPM::TAG_SOURCERPM])
      p.add_int32(RPM::TAG_BUILDTIME,source[RPM::TAG_BUILDTIME])
      if rpmfile.is_a?(String)
        source = nil
        p.filename = File.basename(rpmfile)
        p.size = File.size(rpmfile)
      end
      p
    end

    def create_bone(rpmfile)
      MPH::Scanpackages.create_bone(rpmfile)
    end

    def update_cache_file(f)
      lock
      mph = "#{@cachedir}/#{File.basename(f)}.mph"
      mphrpm = "#{@cachedir}/#{File.basename(f)}.mphrpm"
      begin
        Dir.mkdir(@cachedir, 0775 )
        File.chmod( 0775, @cachedir)
      rescue
      end
      sTime = File.mtime(f)
      p = nil
      if !(File.exist?(mph) && File.exist?(mphrpm) &&
           (File.mtime(mph) > sTime) && (File.mtime(mphrpm) > sTime))
        p = create_bone(f)
        io = open(mph,'w')
        io.print(p.traditional_dump)
        io.close()
        io = open(mphrpm,'w')
        io.print("#{p.filename}\n")
        io.print("#{p.size}\n")
        Marshal.dump(p,io)
        io.close()
      end
      p
    ensure
      unlock
    end

    def update_cache_dir(test_mode)
      rpms = {}
      Dir.glob("#{@dir}/*.rpm").each do |f|
        @msg.print(MsgLevel::VERBOSE, "Update cachefile for #{File.basename(f)} .. ")
        rpms[f] = true
        if update_cache_file(f)
          @msg.print(MsgLevel::VERBOSE, "Done.\n")
        else
          @msg.print(MsgLevel::VERBOSE, "Skip.\n")
        end
      end
      Dir.glob("#{@cachedir}/*.rpm.mph*").each do |f|
        rpm = "#{@dir}/#{File.basename(f,".*")}"
        File.unlink(f) if !rpms[rpm]
      end
    end
    private :update_cache_dir

    def merge_db(suffix,tmpfile)
      Dir.glob("#{@cachedir}/*.rpm.#{suffix}").each do |f|
        File.open(f) do |cache|
          tmpfile.write(cache.read)
        end
      end
      tmpfile.close
    end
    private :merge_db

    def create_db
      merge_db("mph",File.open("#{@dir}.mph_","w"))
      File.unlink("#{@dir}.mph") if File.exist?("#{@dir}.mph")
      File.rename("#{@dir}.mph_","#{@dir}.mph")
      if Zlib then
        merge_db("mphrpm",Zlib::GzipWriter.open("#{@dir}.mphrpm_"))
        File.unlink("#{@dir}.mphrpm") if File.exist?("#{@dir}.mphrpm")
        File.rename("#{@dir}.mphrpm_","#{@dir}.mphrpm")
      end
    end
    private :create_db

    def self.scan(dir,wait_mode = true,remove_cache = false,test_mode = false)
      scanpackages = self.new(dir,wait_mode)
      scanpackages.scan(remove_cache,test_mode)
    end

    def scan(remove_cache = false,test_mode = false)
      lock
      if !test_mode
        if remove_cache
          @msg.print(MsgLevel::VERBOSE,"Remove cachedir #{@cachedir}\n")
          `rm -rf #{@cachedir}`
        end
        begin
          @msg.print(MsgLevel::VERBOSE,"Create cachedir #{@cachedir}\n")
          Dir.mkdir(@cachedir, 0775 )
          File.chmod( 0775, @cachedir)
        rescue
        end
      end
      update_cache_dir(test_mode)
      @msg.print(MsgLevel::VERBOSE,"Create database for #{@dir}\n")
      create_db if !test_mode
    ensure
      unlock
    end  
    public :scan

    def lock
      if @lockcount == 0
        @lockfile = open(@lockfilename, 'w' )
        begin
          if @wait_mode
            @lockfile.setlkw( File::F_WRLCK )
          else
            @lockfile.setlk( File::F_WRLCK )
          end
        rescue
          @msg.print(MsgLevel::ERROR,"failed to lock #{@lockfilename}\n")
          exit( -1 )
        end
      end
      @lockcount += 1
    end
    private :lock

    def unlock
      @lockcount -= 1
      if @lockcount == 0
        @lockfile.close()
        File.delete(@lockfilename)
      end
    end
    private :unlock

    def initialize(dir,wait_mode = true)
      @msg = Msg.instance
      @dir = dir
      @cachedir = "#{File.dirname(dir)}/.#{File.basename(dir)}.mphcache"
      @lockfilename = "#{File.dirname(dir)}/.#{File.basename(dir)}.mph.lock"
      @lockfile = nil
      @lockcount = 0
      @wait_mode = wait_mode
    end
  end
end

if __FILE__ == $0 then
  require 'parsearg'

  def m_ShowUsage()
    print( "Usage:\n" )
    print( "\t#{$0} [-Vnvrdw] binarydir [binarydir ..]\n")
    print( "\t  -V\tprint version\n" )
    print( "\t  -n\tdon't execute\n" )
    print( "\t  -v\tverbose mode\n" )
    print( "\t  -r\tignore cache, and remake database from scratch\n" )
    print( "\t  -w\twait until lockfile is unlocked\n" )
  end

  parseArgs( 0, nil, 'nvrwV' )

  if( $OPT_V == true)
    print( "0.92.7\n" )
    exit( 0 )
  end

  if( ARGV.size == 0 )
    m_ShowUsage()
    exit( 0 )
  end

  if $OPT_v
    MPH::Msg.instance.level = MPH::MsgLevel::VERBOSE
  end

  ARGV.each do |sDir|
    MPH::Scanpackages.scan(File.expand_path(sDir),$OPT_w,$OPT_r,$OPT_n)
  end
end

### :nodoc:
### Local Variables:
### mode: ruby
### ruby-indent-level: 2
### indent-tabs-mode: nil
### End:
