require 'mph'

module RPM
  class Package
    def traditional_dump_dependency(array)
      str = ""
      if array && array[0]
        if array[0].is_a? RPM::Require
          str = "Depends"
        elsif array[0].is_a? RPM::Provide
          str = "Provides"
        elsif array[0].is_a? RPM::Conflict
          str = "Conflicts"
        elsif array[0].is_a? RPM::Obsolete
          str = "Obsoletes"
        end

        if str != ""
          vstr = str + "Ver:"
          fstr = str + "Flags:"
          str << ":"
          array.each do |d|
            str << " " << d.name
            vstr << " ["
            if d.version.e
              vstr << d.version.e.to_s << ":"
            end
            vstr << d.version.to_s<< "]"
            fstr << " " << d.flags.to_s
          end
          str << "\n" << vstr << "\n" << fstr << "\n" 
        end
      end
      str
    end
    private :traditional_dump_dependency

    def traditional_dump
      str = ""
      str << "Package: " << name << "\n"
      str << "Version: " << version.to_s << "\n"
      str << traditional_dump_dependency(requires)
      str << traditional_dump_dependency(obsoletes)
      str << traditional_dump_dependency(provides)
      str << traditional_dump_dependency(conflicts)
      str << "Filename: " << @filename << "\n"
      str << "Size: " << @size.to_s << "\n"
      if signature
        str << "SigMD5: #{signature.id2name}"
      end
#      if sigmd5
#        str << "SigMD5: "
#        sigmd5.each_byte do |byte|
#          str << "%02x"%byte
#        end
#      end
      if version.e
        str << "\nEpoch: #{version.e}"
      end
      str << "\n"
      str << "\n"
    end

    def self.from_traditional_depname_add(p,tag,str)
      str.split( / / ).each do |s|
        p.add_string_array(tag,s)
      end
    end

    def self.from_traditional_depversion_add(p,tag,str)
      str.split( / / ).each do |s|
        s.gsub!( /[\[\]]/ ,"")
        p.add_string_array(tag,s)
      end
    end

    def self.from_traditional_depflag_add(p,tag,str)
      str.split( / / ).each do |s|
        p.add_int32(tag,s.to_i)
      end
    end

    def self.from_traditional(aas)
      name = nil
      version = nil
      epoch = nil
      p = nil
      aas.each do |as|
        case as[0]
        when 'Package:'
          name = as[1]
        when 'Version:'
          version = as[1]
        when 'Epoch:'
          epoch = as[1]
        end
      end

      if name && version
        if( epoch != nil )
          version = RPM::Version.new(version,epoch)
        else
          version = RPM::Version.new(version)
        end
        p = RPM::Package.create(name,version)

        aas.each do |as|
          case as[0]
          when 'Depends:'
            from_traditional_depname_add(p,RPM::TAG_REQUIRENAME,as[1])
          when 'DependsVer:'
            from_traditional_depversion_add(p,RPM::TAG_REQUIREVERSION,as[1])
          when 'DependsFlags:'
            from_traditional_depflag_add(p,RPM::TAG_REQUIREFLAGS,as[1])
          when 'Obsoletes:'
            from_traditional_depname_add(p,RPM::TAG_OBSOLETENAME,as[1])
          when 'ObsoletesVer:'
            from_traditional_depversion_add(p,RPM::TAG_OBSOLETEVERSION,as[1])
          when 'ObsoletesFlags:'
            from_traditional_depflag_add(p,RPM::TAG_OBSOLETEFLAGS,as[1])
          when 'Provides:'
            from_traditional_depname_add(p,RPM::TAG_PROVIDENAME,as[1])
          when 'ProvidesVer:'
            from_traditional_depversion_add(p,RPM::TAG_PROVIDEVERSION,as[1])
          when 'ProvidesFlags:'
            from_traditional_depflag_add(p,RPM::TAG_PROVIDEFLAGS,as[1])
          when 'Conflicts:'
            from_traditional_depname_add(p,RPM::TAG_CONFLICTNAME,as[1])
          when 'ConflictsVer:'
            from_traditional_depversion_add(p,RPM::TAG_CONFLICTVERSION,as[1])
          when 'ConflictsFlags:'
            from_traditional_depflag_add(p,RPM::TAG_CONFLICTFLAGS,as[1])
          when 'Filename:'
            p.filename = as[1].sub( %r|.*/|, '' )
          when 'Size:'
            p.size = Integer( as[1] )
          when 'SigMD5:'
            sigmd5 = ""
            0.step(30,2) do |i|
              sigmd5 << as[1][i,2].hex.chr
            end
            p.add_binary(RPM::TAG_SIGMD5,sigmd5)
          end
        end
      end
      p
    end
  end
end

module MPH
  class TraditionalDBLoader<DBLoader
    def parse_dbfile(io,location)
      lines = []
      io.each_line do |l|
        asData = l.chomp.split( / / ,2)
        if( asData[0] == nil || asData[0] == "")
          if (lines.length() > 0)
            package = RPM::Package.from_traditional(lines)
            return false if !package
            package.location = location
            @packages << package
          end
          lines = []
        else
          lines << asData
        end
      end
      true
    end

    def initialize(dbpath,localpath,dont_get_dbfile = false)
      super(dbpath,localpath,dont_get_dbfile,".mph",File)
    end
  end
end
